"""Test `nth` last child selectors."""
from .. import util


class TestNthLastChild(util.TestCase):
    """Test `nth` last child selectors."""

    def test_nth_child_of_s_complex(self):
        """Test `nth` child with selector (complex)."""

        markup = """
        <body>
        <p id="0"></p>
        <p id="1"></p>
        <span id="2"></span>
        <span id="3"></span>
        <span id="4"></span>
        <span id="5"></span>
        <span id="6"></span>
        <p id="7"></p>
        <p id="8"></p>
        <p id="9"></p>
        <p id="10"></p>
        <span id="11"></span>
        </body>
        """

        self.assert_selector(
            markup,
            ":nth-last-child(2n + 1 of p[id], span[id])",
            ['1', '3', '5', '7', '9', '11'],
            flags=util.HTML
        )



"""Test defined selectors."""
from .. import util


class TestDefined(util.TestCase):
    """Test defined selectors."""

    def test_defined_html(self):
        """Test defined HTML."""

        markup = """
        <!DOCTYPE html>
        <html>
        <head>
        </head>
        <body>
        <div id="0"></div>
        <div-custom id="1"></div-custom>
        <prefix:div id="2"></prefix:div>
        <prefix:div-custom id="3"></prefix:div-custom>
        </body>
        </html>
        """

        self.assert_selector(
            markup,
            'body :defined',
            ['0', '2', '3'],
            flags=util.HTML
        )

    @util.skip_no_lxml
    def test_defined_xhtml(self):
        """Test defined XHTML."""

        markup = """
        <?xml version="1.0" encoding="UTF-8"?>
        <!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN"
            "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
        <html lang="en" xmlns="http://www.w3.org/1999/xhtml">
        <head>
        </head>
        <body>
        <div id="0"></div>
        <div-custom id="1"></div-custom>
        <prefix:div id="2"></prefix:div>
        <!--
        lxml seems to strip away the prefix in versions less than 4.4.0.
        This was most likely because prefix with no namespace is not really valid.
        XML does allow colons in names, but encourages them to be used for namespaces.
        This is a quirk of LXML, but it appears to be fine in 4.4.0+.
        -->
        <prefix:div-custom id="3"></prefix:div-custom>
        </body>
        </html>
        """

        from lxml import etree

        self.assert_selector(
            markup,
            'body :defined',
            # We should get 3, but for LXML versions less than 4.4.0 we don't for reasons stated above.
            ['0', '2'] if etree.LXML_VERSION < (4, 4, 0, 0) else ['0', '1', '2'],
            flags=util.XHTML
        )

    def test_defined_xml(self):
        """Test defined HTML."""

        markup = """
        <?xml version="1.0" encoding="UTF-8"?>
        <html>
        <head>
        </head>
        <body>
        <div id="0"></div>
        <div-custom id="1"></div-custom>
        <prefix:div id="2"></prefix:div>
        <prefix:div-custom id="3"></prefix:div-custom>
        </body>
        </html>
        """

        # Defined is a browser thing.
        # XML doesn't care about defined and this will match nothing in XML.
        self.assert_selector(
            markup,
            'body :defined',
            [],
            flags=util.XML
        )



"""Test indeterminate selectors."""
from .. import util


class TestIndeterminate(util.TestCase):
    """Test indeterminate selectors."""

    def test_indeterminate(self):
        """Test indeterminate."""

        markup = """
        <input type="radio" name="" id="radio-no-name1">
        <label>No name 1</label>
        <input type="radio" name="" id="radio-no-name2" checked>
        <label>no name 2</label>
        <div>
          <input type="checkbox" id="checkbox" indeterminate>
          <label for="checkbox">This label starts out lime.</label>
        </div>
        <div>
          <input type="radio" name="test" id="radio1">
          <label for="radio1">This label starts out lime.</label>
          <form>
            <input type="radio" name="test" id="radio2">
            <label for="radio2">This label starts out lime.</label>

            <input type="radio" name="test" id="radio3" checked>
            <label for="radio3">This label starts out lime.</label>

            <input type="radio" name="other" id="radio4">
            <label for="radio4">This label starts out lime.</label>

            <input type="radio" name="other" id="radio5">
            <label for="radio5">This label starts out lime.</label>
          </form>
          <input type="radio" name="test" id="radio6">
          <label for="radio6">This label starts out lime.</label>
        </div>
        """

        self.assert_selector(
            markup,
            ":indeterminate",
            ['checkbox', 'radio1', 'radio6', 'radio4', 'radio5', 'radio-no-name1'],
            flags=util.HTML
        )

    def test_iframe(self):
        """Test indeterminate when `iframe` is involved."""

        markup = """
        <form>
            <input type="radio" name="test" id="radio1">
            <label for="radio1">This label starts out lime.</label>

            <iframe>
            <html>
            <body>
            <input type="radio" name="test" id="radio2" checked>
            <label for="radio2">This label starts out lime.</label>

            <input type="radio" name="other" id="radio3">
            <label for="radio3">This label starts out lime.</label>
            </body>
            </html>
            </iframe></form>"""

        self.assert_selector(
            markup,
            ":indeterminate",
            ['radio1', 'radio3'],
            flags=util.PYHTML
        )



"""Test paused selectors."""
from .. import util


class TestPaused(util.TestCase):
    """Test paused selectors."""

    MARKUP = """
    <!DOCTYPE html>
    <html>
    <body>

    <video id="vid" width="320" height="240" controls>
      <source src="movie.mp4" type="video/mp4">
      <source src="movie.ogg" type="video/ogg">
      Your browser does not support the video tag.
    </video>

    </body>
    </html>
    """

    def test_paused(self):
        """Test paused (matches nothing)."""

        # Not actually sure how this is used, but it won't match anything anyways
        self.assert_selector(
            self.MARKUP,
            "video:paused",
            [],
            flags=util.HTML
        )

    def test_not_paused(self):
        """Test not paused."""

        self.assert_selector(
            self.MARKUP,
            "video:not(:paused)",
            ["vid"],
            flags=util.HTML
        )



"""Test class selectors."""
from .. import util
from soupsieve import SelectorSyntaxError


class TestClass(util.TestCase):
    """Test class selectors."""

    MARKUP = """
    <div>
    <p>Some text <span id="1" class="foo"> in a paragraph</span>.
    <a id="2" class="bar" href="http://google.com">Link</a>
    </p>
    </div>
    """

    # Browsers normally replace NULL with `\uFFFD`, but some of the parsers
    # we test just strip out NULL, so we will simulate and just insert `\uFFFD` directly
    # to ensure consistent behavior in our tests across parsers.
    MARKUP_NULL = """
    <div>
    <p>Some text <span id="1" class="foo\ufffd"> in a paragraph</span>.
    <a id="2" class="\ufffdbar" href="http://google.com">Link</a>
    </p>
    </div>
    """

    def test_class(self):
        """Test class."""

        self.assert_selector(
            self.MARKUP,
            ".foo",
            ["1"],
            flags=util.HTML
        )

    def test_type_and_class(self):
        """Test type and class."""

        self.assert_selector(
            self.MARKUP,
            "a.bar",
            ["2"],
            flags=util.HTML
        )

    def test_type_and_class_escaped_null(self):
        """Test type and class with an escaped null character."""

        self.assert_selector(
            self.MARKUP_NULL,
            r"a.\0 bar",
            ["2"],
            flags=util.HTML
        )

    def test_type_and_class_escaped_eof(self):
        """Test type and class with an escaped EOF."""

        self.assert_selector(
            self.MARKUP_NULL,
            "span.foo\\",
            ["1"],
            flags=util.HTML
        )

    def test_malformed_class(self):
        """Test malformed class."""

        # Malformed class
        self.assert_raises('td.+#some-id', SelectorSyntaxError)

    def test_class_xhtml(self):
        """Test tag and class with XHTML since internally classes are stored different for XML."""

        self.assert_selector(
            self.wrap_xhtml(self.MARKUP),
            ".foo",
            ["1"],
            flags=util.XHTML
        )

    def test_multiple_classes(self):
        """Test multiple classes."""

        markup = """
        <div>
        <p>Some text <span id="1" class="foo"> in a paragraph</span>.
        <a id="2" class="bar" href="http://google.com">Link</a>
        <a id="3" class="foo" href="http://google.com">Link</a>
        <a id="4" class="foo bar" href="http://google.com">Link</a>
        </p>
        </div>
        """

        self.assert_selector(
            markup,
            "a.foo.bar",
            ["4"],
            flags=util.HTML
        )

    def test_malformed_pseudo_class(self):
        """Test malformed class."""

        # Malformed pseudo-class
        self.assert_raises('td:#id', SelectorSyntaxError)
